-- Maintenance Madness - 2019. Created by Arcitos. License: See mod page.

--- CONSTANTS

local const = {}

-- control flag: Is maintenance for this map enabled?
const.maintenanceActivated = settings.global["mm-mod-active"].value
-- cost level, higher levels increase difficulty
const.maintenanceLevel = settings.global["mm-maintenance-cost"].value

const.costFactor = {
	["level-0"] = 0.5,
	["level-1"] = 1.0,
	["level-2"] = 1.5,
	["level-3"] = 2.5}
	
-- mean time between maintenance events
const.MTBM = math.floor(3600 * settings.global["mm-mean-time-between-maintenance-events"].value) -- 1 min
-- mean time to maintain
const.MTTM = math.floor(3600 * settings.global["mm-mean-time-to-maintain"].value) -- 0,5 min
-- mean time to repair
const.MTTR = math.floor(3600 * settings.global["mm-mean-time-to-repair"].value) -- 0,5 min

const.showText = settings.global["mm-show-flying-text"].value

const.showMIPsignForSolar = settings.global["mm-show-maintenance-in-progress-sign-for-solar"].value
-- cycle time multiplier
const.solarCycleTimeMultiplier = settings.global["mm-solar-factor"].value

const.maxAge = settings.global["mm-max-age"].value

const.replacementAge = settings.global["mm-replacement-age"].value

const.baseAgeing = settings.global["mm-base-ageing"].value

const.maxMissedMaintenanceMalus = settings.global["mm-missed-maintenance-malus"].value

const.repairAgeMalus = const.baseAgeing + const.maxMissedMaintenanceMalus * 0,5

const.repairTimeModifier = 1.50 -- the older the machine, the longer the repair time. Repair time = MTTR + (Age/MaxAge * repairTimeModifier * MTTR)

const.repairProbabilityModifier = 0.50 -- Repair probability = Age/MaxAge * repairProbabilityModifier

const.tickDelayCheckRepairState = 1000 -- check every 16,6 sec

const.idleWearReductionFactor = 0.80 -- WIP

const.baseDamageFactor = 0.15

const.maxDamageFactor = 0.85 -- base damage + max damage = critical, if age > 100%

const.freeCycles = 6 -- number of bonus cycles without maintenance granted to every newly placed entity 

const.forcesWithMROdisabled = {
	["neutral"] = true, 
	["enemy"] = true, 
	["enemy-primitive"] = true, 
	["enemy-easy"] = true, 
	["enemy-medium"] = true, 
	["enemy-advanced"] = true}

const.entityTypesWithMROenabled = {
	["assembling-machine"] = true, 
	["furnace"] = true, 
	["solar-panel"] = settings.global["mm-include-solar-panels"].value, 
	["accumulator"] = settings.global["mm-include-accumulators"].value, 
	["reactor"] = settings.global["mm-include-reactors"].value,
	["boiler"] = settings.global["mm-include-boiler-and-generators"].value,
	["generator"] = settings.global["mm-include-boiler-and-generators"].value,
	["mining-drill"] = settings.global["mm-include-miners"].value,
	["roboport"] = settings.global["mm-include-roboports"].value,
	["beacon"] = settings.global["mm-include-beacons"].value,
	["lab"] = settings.global["mm-include-labs"].value}

-- prio 1
const.entityMaintencanceDemandByName = {}

-- prio 2
const.entityMaintenanceDemandByType = {
	["assembling-machine"] = {	
		{name = "electronic-circuit", amount = 2, probability = 0.2},
		{name = "iron-gear-wheel", amount = 1, probability = 0.75}, 
		{name = "pipe", amount = 1, probability = 0.4},
		{name = "mm-machine-oil", amount = 3, probability = 1},
		{name = "mm-detergent", amount = 1, probability = 0.8}},
	["furnace"] = {				
		{name = "iron-gear-wheel", amount = 2, probability = 0.5},
		{name = "stone-brick", amount = 2, probability = 0.8},
		{name = "mm-machine-oil", amount = 2, probability = 0.8},
		{name = "mm-detergent", amount = 1, probability = 1}},
	["solar-panel"] = {
		{name = "electronic-circuit", amount = 1, probability = 0.1},
		{name = "copper-cable", amount = 1, probability = 0.25},
		{name = "mm-detergent", amount = 1, probability = 1}},
	["accumulator"] = {
		{name = "electronic-circuit", amount = 1, probability = 0.2},
		{name = "copper-cable", amount = 1, probability = 0.75},
		{name = "mm-detergent", amount = 1, probability = 1}},
	["reactor"] = {
		{name = "mm-machine-oil", amount = 10, probability = 0.8},
		{name = "mm-detergent", amount = 15, probability = 0.9},
		{name = "mm-toolbox", amount = 3, probability = 1},
		{name = "copper-cable", amount = 25, probability = 0.75},
		{name = "copper-plate", amount = 12, probability = 0.2},
		{name = "advanced-circuit", amount = 3, probability = 0.3},
		{name = "pipe", amount = 12, probability = 0.4}},
	["boiler"] = {
		{name = "pipe", amount = 2, probability = 0.8},
		{name = "stone-brick", amount = 2, probability = 0.5},
		{name = "mm-machine-oil", amount = 2, probability = 0.6},
		{name = "mm-detergent", amount = 1, probability = 1}},
	["generator"] = {
		{name = "pipe", amount = 3, probability = 0.3},
		{name = "iron-gear-wheel", amount = 2, probability = 0.8},
		{name = "copper-cable", amount = 2, probability = 0.75},
		{name = "mm-machine-oil", amount = 8, probability = 1},
		{name = "mm-detergent", amount = 1, probability = 0.75}},
	["mining-drill"] = {
		{name = "steel-plate", amount = 1, probability = 0.8},
		{name = "iron-stick", amount = 4, probability = 0.75},
		{name = "iron-gear-wheel", amount = 1, probability = 0.3},
		{name = "mm-machine-oil", amount = 3, probability = 1}},
	["roboport"] = {		
		{name = "iron-gear-wheel", amount = 2, probability = 0.8},
		{name = "electronic-circuit", amount = 2, probability = 0.7},
		{name = "advanced-circuit", amount = 1, probability = 0.4},
		{name = "mm-toolbox", amount = 1, probability = 0.2},
		{name = "mm-detergent", amount = 8, probability = 1},
		{name = "mm-machine-oil", amount = 4, probability = 0.7}},
	["beacon"] = {
		{name = "iron-gear-wheel", amount = 2, probability = 0.3},
		{name = "electronic-circuit", amount = 1, probability = 0.9},
		{name = "advanced-circuit", amount = 1, probability = 0.6},
		{name = "copper-cable", amount = 3, probability = 0.75},
		{name = "mm-detergent", amount = 5, probability = 0.7},
		{name = "mm-machine-oil", amount = 5, probability = 1}},
	["lab"] = {
		{name = "electronic-circuit", amount = 1, probability = 0.2},
		{name = "iron-stick", amount = 5, probability = 0.6},
		{name = "copper-cable", amount = 3, probability = 0.5},
		{name = "mm-detergent", amount = 12, probability = 1}}
}

const.textColors = {
		maintenance = {
			min = {r=1, g=0.48, b=0.15, a=0.8}, -- a = 0.5
			max = {r=0.7, g=0.7, b=0.7, a=0.8}, -- a = 0.5
			delta = {r=-0.3, g=0.22, b=0.55, a=0.0}
		},
		failure = {r=1, g=0.3, b=0.2, a=1}, -- a = 0.9
		failure2 = {r=1, g=0.4, b=0.26, a=0.8}, -- a = 0.5
		repair = {r=0.14, g=0.5, b=0.32, a=0.8}
	}

return const