-- Maintenance Madness - 2019. Created by Arcitos. License: See mod page.

local mm_util = require("util.util")
local internalSettings = require("config-startup")

local recycleTimeMultiplier = internalSettings.recycleTimeMultiplier
local reconditionTimeMultiplier = internalSettings.reconditionTimeMultiplier

for prototypeGroup, _ in pairs(internalSettings.entityTypesWithMROenabled) do
	--log("Processed group: "..prototypeGroup)
	for __, entityPrototype in pairs(data.raw[prototypeGroup]) do
		
		local protName = entityPrototype.name
		--log("Processed: "..protName)
		local recipe = nil
		local item = nil
		local useRotatedRequesterContainer = false
		if string.sub(protName, 1, 10) == "mm-faulty-" then
			goto continue 
			-- Ignore faulty versions of solar-panels or accumulators created by this mod
		end		
		if data.raw.recipe[protName] then
			recipe = data.raw.recipe[protName]
		else
			-- If no recipe for an entity is found, there is no way to determine repair costs - skip this entity.
			log("Missing recipe for: "..protName..". Maintenance for this entity will be disabled.")
			goto continue 
		end
		if data.raw.item[protName] then
			item = data.raw.item[protName]
		else
			-- If no recipe for an entity is found, there is no way to create an scrap item - skip this entity.
			log("Missing item for: "..protName..". Maintenance for this entity will be disabled.")
			goto continue
		end	
		
		local itemIcon
		if item.icons then
			itemIcon = table.deepcopy(item.icons)
			-- TODO: Multi icon entities
		else
			local itemIconScale = 32 / item.icon_size * 0.9 -- bigger icon sizes will be resized to fit into 32x32
			itemIcon = {
				{icon = "__base__/graphics/icons/remnants.png", tint = {a = 1, r = 0.8, g = 0.4, b = 0.3}, icon_size = 32},
				{icon = item.icon, scale = itemIconScale, tint = {a = 1, r = 0.8, g = 0.4, b = 0.3}, icon_size = 32}
			}
		end	
		
		local scrappedItem = {
		-- Basic info
			type = "item",
			name = "mm-scrapped-"..protName,
			localised_name = {"mm-item-name-scrap", {"entity-name."..protName}},
			--icon = item.icon,
			icons = itemIcon,
			icon_size = item.icon_size,
			flags = {},
			subgroup = "mm-scrap",
			order = item.order,
			stack_size = item.stack_size,
			allow_decomposition = false
		}
		
		local timeNeeded = recipe.energy_required or 0.5
		local recyclingIcon = table.deepcopy(itemIcon)
		table.insert(recyclingIcon, {icon = "__MaintenanceMadness__/graphics/icons/recycle-32px.png", scale = 0.7, icon_size = 32})
		
		local recyclingRecipe = {
			type = "recipe",
			name = "mm-recycle-"..protName,
			localised_name = {"mm-recipe-name-recyling", {"entity-name."..protName}},
			category = "mm-recycling",
			icons = recyclingIcon,
			icon_size = item.icon_size,
			subgroup = "mm-recyling",
			order = item.order
		}	
		local recyclingRecipeSpecs = {
			enabled = true,
			hidden = true,
			results = {},
			ingredients = {{"mm-scrapped-"..protName, 1}},
			energy_required = math.max(timeNeeded * recycleTimeMultiplier, internalSettings.recycleTimeMinimum),
			allow_decomposition = false,
			allow_as_intermediate = false,
			allow_intermediates = false,
			always_show_products = true,
			show_amount_in_title = false,
			main_product = nil
		}
		
		local reconditionIcon = table.deepcopy(itemIcon)
		table.insert(reconditionIcon, {icon = "__base__/graphics/icons/repair-pack.png", scale = 0.7, icon_size = 32})
		
		local reconditionRecipe = {
			type = "recipe",
			name = "mm-recondition-"..protName,
			localised_name = {"mm-recipe-name-recondition", {"entity-name."..protName}},
			--category = "centrifuging",
			icons = reconditionIcon,
			icon_size = item.icon_size,
			subgroup = "mm-recondition",
			order = item.order
		}	
		local reconditionRecipeSpecs = {
			enabled = false,
			results = {{type = "item", name = protName, amount = 1}},		
			ingredients = {{"mm-scrapped-"..protName, 1}, {"mm-toolbox", 1}},
			energy_required = math.max(timeNeeded * reconditionTimeMultiplier, internalSettings.reconditionTimeMinimum),
			allow_decomposition = false,
			allow_as_intermediate = false,
			allow_intermediates = false,
			always_show_products = true,
			show_amount_in_title = false,
			main_product = nil
		}
		
		if recipe.normal ~= nil or recipe.expensive ~= nil then
		-- If this is a recipe with "normal" and/or "expensive" cost, create normal/expensive recipe cost for the recycling/recondition items, too
			if recipe.normal then
				recyclingRecipe.normal = table.deepcopy(recyclingRecipeSpecs)
				reconditionRecipe.normal = table.deepcopy(reconditionRecipeSpecs)
				for _, ingredient in pairs(recipe.normal.ingredients) do
					local amount = mm_util.getIngredientAmount(ingredient)
					if ingredient.type ~= "fluid" and amount >= 2 then 	
						table.insert(recyclingRecipe.normal.results, {
							type = "item",
							name = ingredient.name or ingredient[1],
							amount_min = math.floor(amount/5),
							amount_max = math.floor(amount/2)
						})
						table.insert(reconditionRecipe.normal.ingredients, {
							type = "item",
							name = ingredient.name or ingredient[1],
							amount = math.floor(amount * internalSettings.reconditionSparePartFactor),
						})
					end		
				end	
			else
				recyclingRecipe.normal = recipe.normal -- nil or false
				reconditionRecipe.normal = recipe.normal
			end
			if recipe.expensive then
				recyclingRecipe.expensive = table.deepcopy(recyclingRecipeSpecs)
				reconditionRecipe.expensive = table.deepcopy(reconditionRecipeSpecs)
				for _, ingredient in ipairs(recipe.expensive.ingredients) do
					local amount = mm_util.getIngredientAmount(ingredient)
					if ingredient.type ~= "fluid" and amount >= 2 then 	
						table.insert(recyclingRecipe.expensive.results, {
							type = "item",
							name = ingredient.name or ingredient[1],
							amount_min = math.floor(amount/5),
							amount_max = math.floor(amount/2)
						})
						table.insert(reconditionRecipe.expensive.ingredients, {
							type = "item",
							name = ingredient.name or ingredient[1],
							amount = math.floor(amount * internalSettings.reconditionSparePartFactor),
						})				
					end		
				end
			else
				recyclingRecipe.expensive = recipe.expensive -- nil or false
				reconditionRecipe.expensive = recipe.expensive
			end
		else
			mm_util.appendTable(recyclingRecipe, table.deepcopy(recyclingRecipeSpecs))
			mm_util.appendTable(reconditionRecipe, table.deepcopy(reconditionRecipeSpecs))
			for _, ingredient in ipairs(recipe.ingredients) do
				local amount = mm_util.getIngredientAmount(ingredient)
				if ingredient.type ~= "fluid" and amount >= 2 then 		
					-- Recondition demands 50% of the initial spent resources. This implies an item amount > 1.
					table.insert(recyclingRecipe.results, {
						type = "item",
						name = ingredient.name or ingredient[1],
						amount_min = math.floor(amount/5),
						amount_max = math.floor(amount/2)
					})
					-- Recondition demands a certain percentage of the initial spent resources.
					table.insert(reconditionRecipe.ingredients, {
						type = "item",
						name = ingredient.name or ingredient[1],
						amount = math.floor(amount * internalSettings.reconditionSparePartFactor),
					})				
				end		
			end
		end
		
		local dummy = {
			filename = "__MaintenanceMadness__/graphics/dummy.png",
			priority = "high",
			width = 1,
			height = 1,
			frame_count = 1
		}
		local requesterContainer = {
		-- This chest is required as a target for item request proxies
			type = "container",
			name = "mm-chest-"..protName,
			icon = "__base__/graphics/icons/wooden-chest.png",
			icon_size = 32,
			flags = {"placeable-neutral", "player-creation", "placeable-off-grid", "not-on-map", "not-blueprintable", "not-deconstructable", "hide-alt-info", "no-automated-item-removal", "no-automated-item-insertion"},
			--minable = false,
			max_health = 10000, -- invincible, removed only via script
			healing_per_tick = 10000,
			corpse = "small-remnants",
			collision_box = entityPrototype.collision_box,
			selection_box = entityPrototype.selection_box,
			collision_mask = {"not-colliding-with-itself"},
			order="z",
			alert_icon_shift = entityPrototype.alert_icon_shift,
			inventory_size = 8,
			open_sound = { filename = "__base__/sound/wooden-chest-open.ogg" },
			close_sound = { filename = "__base__/sound/wooden-chest-close.ogg" },
			vehicle_impact_sound =  { filename = "__base__/sound/car-wood-impact.ogg", volume = 1.0 },
			picture = dummy,
			create_ghost_on_death = false,
			allow_copy_paste = false
		}
		local dimensions = mm_util.getBoundingBoxDimensions(requesterContainer.selection_box)
		-- If an entity hat a non square base, add a rotated version of the maintenance chest
		local requesterContainerRotated = {}
		if dimensions.width ~= dimensions.height then
			useRotatedRequesterContainer = true -- Flag, evaluated later on
			
			requesterContainerRotated = table.deepcopy(requesterContainer)
			requesterContainerRotated.name = "mm-chest-rotated-"..protName			
			requesterContainerRotated.collision_box = mm_util.rotateBoundingBox(requesterContainer.collision_box)
			requesterContainerRotated.selection_box = mm_util.rotateBoundingBox(requesterContainer.selection_box)
		end
		
		--log(serpent.block(requesterContainer, {maxlevel= 1}))
		
		data:extend({
			scrappedItem,
			recyclingRecipe,
			reconditionRecipe,
			requesterContainer			
		})	
		if useRotatedRequesterContainer then
			data:extend({
				requesterContainerRotated
			})
		end
		
		if data.raw.technology["mm-recondition"] then
		-- Add this recipe to the "recondition" tech
			local recipeEffect = {
					type = "unlock-recipe",
					recipe = reconditionRecipe.name}
					
			table.insert(data.raw.technology["mm-recondition"].effects, recipeEffect)
		end
		
		if prototypeGroup == "solar-panel" or prototypeGroup == "accumulator" then
		-- Solar panels and accumulators need special treatment, because simply setting "active" to "false" doesn't do anything. 
		-- With this function, we create "faulty" versions of them, with much less energy output.
		-- If a solar panel or an accumulator breaks down, replace the entity with a "faulty" one to simulate the effect.
			local faultyEntityIcon
			if item.icons then
				faultyEntityIcon = table.deepcopy(item.icons)
				-- TODO: Multi icon entities
			else
				faultyEntityIcon = {
					{icon = item.icon},
					{icon = "__MaintenanceMadness__/graphics/icons/machine-malfunction-small.png"}
				}
			end	
			
			local faultyEntity = table.deepcopy(entityPrototype)
			faultyEntity.name = "mm-faulty-"..protName
			faultyEntity.localised_name = {"mm-faulty-entity-name", {"entity-name."..protName}}
			faultyEntity.icons = faultyEntityIcon
			faultyEntity.icon = nil
			faultyEntity.icon_size = item.icon_size
			if entityPrototype.localised_desctription ~= nil then 
				faultyEntity.localised_desctription = {"entity-description."..protName} 
			end
			if prototypeGroup == "solar-panel" then 
				faultyEntity.production = "5kW"	
			elseif prototypeGroup == "accumulator" then 
				faultyEntity.energy_source.input_flow_limit = "25kW" 
			end
			--faultyEntity.order = "z"
			
			local faultyEntityItem = {
				type = "item",
				name = "mm-faulty-"..protName,
				place_result = "mm-faulty-"..protName,
				localised_name = item.localised_name,
				--icon = item.icon,
				icons = faultyEntityIcon,
				icon_size = item.icon_size,
				flags = {"hidden"},
				subgroup = "mm-faulty-entity",
				order = item.order,
				stack_size = item.stack_size
			}				
			
			data:extend({
				faultyEntity,
				faultyEntityItem
			})
		end
		::continue:: -- lua apparently doesn't know regular continue orders
	end
end